@@ -159,7 +159,7 @@ if (! window._OQAjaxQueryLoaded)
   });
 
   var downloadCtr = 0;
-  $(document).delegate('.OQDownloadCSV,.OQDownloadHTML,.OQDownloadXML','click', function(){
+  $(document).delegate('.OQDownloadCSV,.OQDownloadHTML,.OQDownloadXML,.OQDownloadJSON','click', function(){
     var target = 'download'+(downloadCtr++); 
     var $f = $('.OQform');
     var $panel = $(this).closest('.OQToolsPanel');
@@ -168,6 +168,7 @@ if (! window._OQAjaxQueryLoaded)
     var dat = buildParamMap($f);
     if (/CSV/.test(this.className)) dat.module = 'CSV';
     else if (/XML/.test(this.className)) dat.module = 'XML';
+    else if (/JSON/.test(this.className)) dat.module = 'JSON';
     else dat.module = 'PrinterFriendly';
     if ($panel.find('.OQExportAllResultsInd:checked').length==1) {
       dat.rows_page = 'All';
